from math import pi
from ColorController.helpers import *
import warnings


def invert_rgb(r, g, b):
    """
    Returns the inverse of r, g, b, where r, g, b are each values in range(0,256).
    """
    if any(ch < 0 or ch > 255 for ch in [r, g, b]):
        raise ValueError(f"Inputs for r, g, and b must be integer values between 0 and 255. "
                         f"You entered values {r, g, b} for (r, g, b)")
    if (type(r), type(g), type(b)) != (int, int, int):
        warnings.warn(f"Inputs for r, g, and b must be integer values between 0 and 255. "
                      f"You entered types {(type(r), type(g), type(b))} for (r, g, b)")
    r = 255 - r
    g = 255 - g
    b = 255 - b
    return r, g, b


def rgb_to_hex(red, green, blue):
    """
    Return color as #rrggbb for the given color values. Does not handle unbounded values (bigger than 255).
    """
    if any(ch < 0 or ch > 255 for ch in [red, green, blue]):
        raise ValueError(f"Inputs for red, green, and blue must be integer values between 0 and 255. "
                         f"You entered values {red, green, blue} for (red, green, blue)")
    if (type(red), type(green), type(blue)) != (int, int, int):
        warnings.warn(f"Inputs for red, green, and blue must be integer values between 0 and 255. "
                      f"You entered types {(type(red), type(green), type(blue))} for (r, g, b)")
    return '#%02x%02x%02x' % (int(round(red)), int(round(green)), int(round(blue)))


def hex_to_rgb(hex_str=str):
    """
    Return (red, green, blue) for the color given as #rrggbb. Values are integers only, as they should be.
    """
    hex_str = hex_str.lstrip('#')
    len_hex = len(hex_str)
    if len_hex != 6:
        raise ValueError('Hex code must be 6 digits long.')
    return tuple(int(hex_str[i:i + len_hex // 3], 16) for i in range(0, len_hex, len_hex // 3))


def colorsys_hsv_to_hsv360(colorsys_hsv=tuple):
    """
    Takes an HSV triplet as provided by colorsys, and converts it to match the
    notation used in colornames.txt
    """
    h = colorsys_hsv[0] * 360
    s = colorsys_hsv[1] * 100
    v = (colorsys_hsv[2] / 255) * 100
    return regular_round(h), regular_round(s), regular_round(v)


def hsv360_to_hsvdistance(hsv360=tuple):
    """
    Takes an HSV triplet as provided by colorsys_hsv_to_hsv360(), and converts it to match the
    notation used in the function for calculating distance between colors.
    """
    new_h = (hsv360[0] / 360) * (2 * pi)
    new_s = hsv360[1] / 100
    new_v = hsv360[2] / 100
    return new_h, new_s, new_v



import matplotlib.pyplot as plt
from ColorController.conversions import invert_rgb, rgb_to_hex


def place_title_linebreak(title):
    title = title.split(" ")
    new_title = ""
    line = ""
    for word in title:
        if len(line) + len(word) < 12:
            line += word + " "
            new_title += word + " "
        else:
            new_title += "\n " + word + " "
            line = ""
    return new_title


def set_title_fontsize(title):
    fontsize = 40
    title_len = len(title)
    if title_len > 15:
        fontsize = 50 - title_len
    return fontsize


def show_named_color(color_object, rotatelabels=False, labelfontsize=12, *args, **kwargs):
    if color_object.name.lower() == 'grey':
        rotatelabels = True
        labelfontsize = 4
    data = [1 for code in color_object.hex_code]
    explode = [.05 for d in data]
    plt.pie(data,
            explode=explode,
            labels=color_object.hex_code,
            rotatelabels=rotatelabels,
            textprops={'fontsize': labelfontsize},
            colors=color_object.hex_code,
            )
    plt.title(place_title_linebreak(color_object.name.upper()), fontsize=set_title_fontsize(color_object.name))
    plt.tight_layout()
    plt.show()


def show_coded_color(color_object, *args, **kwargs):
    fig, ax = plt.subplots()
    data = [1 for name in color_object.name] * 2
    list_spacer = [" " for name in color_object.name]
    hex_str = "#" + color_object.hex_code.lstrip('#')
    ax.pie(data,
           colors=[hex_str for name in color_object.name],
           wedgeprops={'linewidth': 3, 'ec': 'black'}
           )
    ax.pie(data,
           labels=[place_title_linebreak(n.replace("_", " ")).upper() for n in color_object.name] + list_spacer,
           textprops={'fontsize': 15},
           rotatelabels=False,
           startangle=90,
           colors=[hex_str for name in color_object.name + list_spacer],
           wedgeprops={'linewidth': 1, 'ec': hex_str}
           )
    plt.title(color_object.hex_code.upper(), fontsize=40)
    r, g, b = color_object.rgb
    text_r, text_g, text_b = invert_rgb(r, g, b)
    if all(abs(ch1-ch2) < 75 for (ch1, ch2) in zip((text_r, text_g, text_b), (r, g, b))):
        text_color = '#000000'
    else:
        text_color = rgb_to_hex(text_r, text_g, text_b)
    ax.text(x=-.65, y=-.15, s=f'Hex Code: {color_object.hex_code}\n'
                              f'RGB: {color_object.rgb}\n'
                              f'HSV: {color_object.hsv}', color=text_color)
    plt.tight_layout()
    plt.show()
